<?php declare(strict_types = 1);
/*
** Zabbix
** Copyright (C) 2001-2023 Zabbix SIA
** Copyright (C) 2024 Cybertrust Japan Co., Ltd.
**
** This program is free software: you can redistribute it and/or modify it under the terms of
** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License along with this program.
** If not, see <https://www.gnu.org/licenses/>.
**/


class CControllerSnapshotRestore extends CController {

	private $type;

	protected function checkInput(): bool {
		$fields = [
			'snapshotid' => 'required|db snapshots.snapshotid'
		];

		$ret = $this->validateInput($fields);

		if (!$ret) {
			$this->setResponse(
				new CControllerResponseData(['main_block' => json_encode([
					'error' => [
						'title' => _('Cannot restore snapshot'),
						'messages' => array_column(get_and_clear_messages(), 'message')
					]
				])])
			);
		}

		return $ret;
	}

	protected function checkPermissions(): bool {

		$snapshotid = $this->getInput('snapshotid');
		$result = DBfetch(DBSelect(
			'select type from snapshots where snapshotid = '.$snapshotid.';'
		));
		$this->type = $result['type'] ?? ZBX_SNAPSHOT_TYPE_NONE;

		switch ($this->type) {
			case ZBX_SNAPSHOT_TYPE_HOST:
			case ZBX_SNAPSHOT_TYPE_TEMPLATE:
				return $this->checkSnapshotAccess();
			default:
				return false;
		}
	}

	private function getImportRules(): array {
		$rules = [
			'host_groups' => [
				'updateExisting' => true,
				'createMissing' => true
			],
			'valueMaps' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			],
			'templateLinkage' => [
				'createMissing' => true,
				'deleteMissing' => true
			],
			'items' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			],
			'discoveryRules' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			],
			'triggers' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			],
			'graphs' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			],
			'httptests' => [
				'updateExisting' => true,
				'createMissing' => true,
				'deleteMissing' => true
			]
		];
		if ($this->type == ZBX_SNAPSHOT_TYPE_HOST) {
			$rules = array_merge($rules, [
				'hosts' => [
					'updateExisting' => true,
					'createMissing' => true
				]
			]);
		}
		elseif ($this->type == ZBX_SNAPSHOT_TYPE_TEMPLATE) {
			$rules = array_merge($rules, [
				'templates' => [
					'updateExisting' => true,
					'createMissing' => true
				],
				'templateDashboards' => [
					'updateExisting' => true,
					'createMissing' => true,
					'deleteMissing' => true
				]
			]);
		}
		return $rules;
	}

	protected function doAction(): void {
		$snapshotid = $this->getInput('snapshotid');
		$result_fetch = DBfetch(DBSelect(
			'select data from snapshots where snapshotid = '.$snapshotid.';'
		));
		$result_import = API::Configuration()->import([
			'format' => CImportReaderFactory::JSON,
			'source' => $result_fetch['data'],
			'rules' => $this->getImportRules()
		]);
		$output = [];

		if ($result_import) {
			$success = ['title' => _('Snapshot restored')];

			if ($messages = get_and_clear_messages()) {
				$success['messages'] = array_column($messages, 'message');
			}

			$output['success'] = $success;
		}
		else {
			CMessageHelper::setErrorTitle(_('Cannot restore snapshot'));

			$output['error'] = [
				'title' => CMessageHelper::getTitle(),
				'messages' => array_column(get_and_clear_messages(), 'message')
			];
		}

		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
	}
}
